/* futebus example: timed non-blocking reader on broadcast channel

   License: CC0 - no rights reserved

   How to use:
   1. create the bus: futebus bus
   2. run the master: ./Bpoll
   3. observe that Bpoll is idling, reporting no new message periodically
   4. run a process to feed in a message: echo hello >>bus/B1
   5. observe that Bpoll reports new message
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <fcntl.h>

int main(int argc, char* argv[])
{
	int fd, flags;

	/* open the channel */
	fd = open("bus/B1", O_APPEND|O_RDWR);
	if (fd < 0) {
		perror("open()");
		exit(1);
	}

	/* set fd non-blocking */
	flags = fcntl(fd, F_GETFL, 0);
	fcntl(fd, F_SETFL, flags | O_NONBLOCK);

	for(;;) {
		char buf[512];
		int n = read(fd, buf, sizeof(buf));
		if (n < 0) {
			if (errno == EAGAIN)
				printf("<no message in bus>\n");
			else
				perror("read()");
		}
		else if (n == 0) {
			printf("EOF on read\n");
			break;
		}
		else if (n > 0) {
			buf[n] = 0;
			fputs(buf, stdout);
		}
		sleep(1);
	}

	perror("poll()");
	exit(1);
}
